import bpy

#-----------------------------------

class NOXCameraPanel(bpy.types.Panel):
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_idname = "noxcamerapanel"
    bl_label = "NOX Camera"
    bl_context = "data"
    COMPAT_ENGINES = {"nox_renderer"}

    
    def draw(self, context):
        cam = context.camera

        row = self.layout.row()
        col = row.column(align=True)
        col.enabled = False
        col.prop(cam.noxData, "active")
        row.operator("camera.camera_set_active")
  
        self.layout.separator()
        row = self.layout.row()
        row.label(text="Focal")
        row = self.layout.row()
        if cam.lens_unit == 'MILLIMETERS':
            row.prop(cam, "lens")
        elif cam.lens_unit == 'DEGREES':
            row.prop(cam, "angle")
        row.prop(cam, "lens_unit", text="")

        row = self.layout.row(align=True)
        row.operator("camera.camera_change_length", text="15").value = 15.0
        row.operator("camera.camera_change_length", text="20").value = 20.0
        row.operator("camera.camera_change_length", text="24").value = 24.0
        row.operator("camera.camera_change_length", text="28").value = 28.0
        row.operator("camera.camera_change_length", text="35").value = 35.0
        row.operator("camera.camera_change_length", text="50").value = 50.0
        row.operator("camera.camera_change_length", text="85").value = 85.0
        row.operator("camera.camera_change_length", text="135").value = 135.0
        
        self.layout.separator()
        row = self.layout.row()
        row.label(text="Shift")
        row = self.layout.row()
        row.prop(cam, "shift_x")
        row.prop(cam, "shift_y")

        
        self.layout.separator()
        row = self.layout.row()
        row.label(text="Exposure")
        row = self.layout.row()
        row.prop(cam.noxData, "iso")
        row = self.layout.row()
        row.prop(cam.noxData, "aperture")
        row = self.layout.row()
        row.prop(cam.noxData, "shutter")

        self.layout.separator()
        row = self.layout.row()
        row.label(text="Depth of field")
        row = self.layout.row()
        row.prop(cam.noxData, "dof_on")
        row = self.layout.row()
        row.prop(cam, "dof_distance")
        
        
        self.layout.separator()
        row = self.layout.row()
        col = row.column(align=True)
        col.label(text="Aperture")
        col.prop(cam.noxData, "aperture")
        col.prop(cam.noxData, "ap_num_blades")
        col.prop(cam.noxData, "ap_angle")
        col.prop(cam.noxData, "ap_blades_radius")
        col.prop(cam.noxData, "ap_round_blades")

        col = row.column(align=True)
        col.label(text="Bokeh")
        col.prop(cam.noxData, "bokeh_ring_balance")
        col.prop(cam.noxData, "bokeh_ring_size")
        col.prop(cam.noxData, "bokeh_flatten")
        col.prop(cam.noxData, "bokeh_vignette")
       
    @classmethod
    def poll(cls, context):
        return (context.object is not None) and (context.scene.render.engine == 'nox_renderer') and (context.object.type=="CAMERA")
    
#-----------------------------------
