import bpy
from math import sin, cos, pi, asin, atan2

#-----------------------------------

class eval_sun_pos(bpy.types.Operator):
    bl_label = "Eval sun position"
    bl_idname = "world.eval_sun_pos"

    def invoke(self, context, event):
        return self.execute(context)
    
    def execute(self, context):
        world = context.scene.world
        res = eval_sun_position(world)
        if res[0]:
            world.noxData.sun_altitude = res[1]
            world.noxData.sun_azimuth = res[2]

        return {'FINISHED'}

#-----------------------------------

def get_day_of_year(month, day):
    res = day
    if month>1:
        res += 31
    if month>2:
        res += 28
    if month>3:
        res += 31
    if month>4:
        res += 30
    if month>5:
        res += 31
    if month>6:
        res += 30
    if month>7:
        res += 31
    if month>8:
        res += 31
    if month>9:
        res += 30
    if month>10:
        res += 31
    if month>11:
        res += 30
    return res

def eval_sun_position(world):
    if world:
        if hasattr(world, "noxData"):
            month = world.noxData.month
            maxday=31
            if month==1 or month==3 or month==5 or month==7 or month==8 or month==10 or month==12:
                maxday=31
            if month==4 or month==6 or month==9 or month==11:
                maxday=30
            if month==2:
                maxday=28
            day = min(world.noxData.day, maxday)
            hour = world.noxData.hour
            minute = world.noxData.minute
            gmt = world.noxData.gmt
            latitude = world.noxData.latitude
            longitude = world.noxData.longitude
            
            longitude_rad = -longitude*pi/180.0
            latitude_rad = -latitude*pi/180.0
            gmt_rad = -gmt*pi/12.0
            
            dayOfYear = get_day_of_year(month, day)
            earthTimeInHours = dayOfYear*24 + hour + minute/60.0;
            
            a = 0.170*sin(4.0*pi*(dayOfYear-80.0)/373.0)
            b = 0.129*sin(2.0*pi*(dayOfYear-8.0)/355.0)
            c1 = 12.0*(gmt_rad-longitude_rad)/pi
            c2 = 12.0*gmt_rad/pi
            solarTimeInHours = earthTimeInHours + a - b + c1
            solarGlobalTimeInHours = earthTimeInHours + a - b + c2
            solarDeclination = 0.4093*sin(2.0*pi*(dayOfYear-81.0)/368.0)
            
            sindec = sin(solarDeclination)
            cosdec = cos(solarDeclination)
            sinlat = sin(latitude_rad)
            coslat = cos(latitude_rad)
            
            solarTimeInHours -= 12
            thetaS =  pi/2 - asin(sinlat*sindec + coslat*cosdec*cos(pi*solarTimeInHours/12.0))
            cosThetaS = cos(thetaS)

            m = coslat*sindec - sinlat*cosdec*cos(pi*solarTimeInHours/12.0)
            l = -cosdec*sin(pi*solarTimeInHours/12.0)
            phiS = atan2(-l,-m)
            
            #world.noxData.sun_altitude = thetaS*180/pi
            #world.noxData.sun_azimuth = phiS*180/pi
            altitude = 90-thetaS*180/pi
            azimuth = 180+phiS*180/pi

            return (True, altitude, azimuth)
    return (False, 0.0, 0.0)

#-----------------------------------

class uncheck_tex_in_nox_env(bpy.types.Operator):
    bl_label = "Uncheck texture"
    bl_idname = "world.uncheck_tex_in_nox_env"
    
    def invoke(self, context, event):
        world = context.scene.world
        if world is None:
            return {'FINISHED'}
        world.noxDataEnv.texture.texID = -1
        return {'FINISHED'}

#-----------------------------------
