import bpy
from render_nox.env.sunsky_ops import eval_sun_position

#-----------------------------------

class NOXSunskyPanel(bpy.types.Panel):
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_idname = "sunskypanel"
    bl_label = "NOX Sunsky"
    bl_context = "world"
    COMPAT_ENGINES = {"nox_renderer"}

    #def draw_header(self, context):
    #    world = context.scene.world
    #    if world:
    #        if hasattr(world, "noxData"):
    #            self.layout.prop(world.noxData, "active", text="")

    def draw(self, context):
        row = self.layout.row()
        row.template_ID(context.scene, "world", new="world.new")
        self.layout.separator()

        world = context.scene.world
        
        if world:
            if hasattr(world, "noxData"):
            
                esp = eval_sun_position(world)
                if esp[0]:
                    sun_altitude = esp[1]
                    sun_azimuth = esp[2]
                else:
                    sun_altitude = world.noxData.sun_altitude
                    sun_azimuth = world.noxData.sun_azimuth
                    
                row = self.layout.row()
                row.prop(world.noxData, "active")
                row = self.layout.row()
                row.prop(world.noxData, "sky_model")
                self.layout.separator()
                self.layout.separator()

                row = self.layout.row()
                row.label(text="Time")
                row = self.layout.row(align=True)
                row.prop(world.noxData, "month")
                row.prop(world.noxData, "day")
                row = self.layout.row(align=True)
                row.prop(world.noxData, "hour")
                row.prop(world.noxData, "minute")
                row = self.layout.row()
                row.prop(world.noxData, "gmt")
                self.layout.separator()
                self.layout.separator()


                row = self.layout.row()
                row.label(text="Scene position")
                row = self.layout.row(align=True)
                row.prop(world.noxData, "longitude")
                row.prop(world.noxData, "latitude")
                self.layout.separator()
                self.layout.separator()

                row = self.layout.row()
                row.label(text="Orientation")
                row = self.layout.row()
                row.prop(world.noxData, "north_angle")
                self.layout.separator()
                self.layout.separator()
                

                row = self.layout.row()
                row.label(text="Sun")
                row = self.layout.row()
                row.prop(world.noxData, "sun_on")
                row = self.layout.row()
                row.prop(world.noxData, "sun_layer2")
                row = self.layout.row()
                row.prop(world.noxData, "sun_size")
                row = self.layout.row()
                row.label(text="Altitude: " + str(int(sun_altitude)))
                #row = self.layout.row()
                row.label(text="Azimuth: " + str(int(sun_azimuth)))
                #row.operator("world.eval_sun_pos", text="Refresh")
                self.layout.separator()
                self.layout.separator()

                row = self.layout.row()
                row.label(text="Atmosphere")
                row = self.layout.row()
                row.prop(world.noxData, "turbidity")
                row = self.layout.row()
                row.prop(world.noxData, "albedo")
                




                
    @classmethod
    def poll(cls, context):
        return (context.scene.render.engine == 'nox_renderer')

#-----------------------------------

class NOXEnvmapPanel(bpy.types.Panel):
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_idname = "envmappanel"
    bl_label = "NOX Envmap"
    bl_context = "world"
    COMPAT_ENGINES = {"nox_renderer"}

    def draw_header(self, context):
        world = context.scene.world
        if world:
            if hasattr(world, "noxDataEnv"):
                if hasattr(world.noxDataEnv, "texture"):
                    self.layout.prop(world.noxDataEnv.texture, "enabled", text="")

    def draw(self, context):
        row = self.layout.row()
        row.template_ID(context.scene, "world", new="world.new")
        self.layout.separator()

        world = context.scene.world
        
        if world:
            if hasattr(world, "noxDataEnv"):
                #if hasattr(world.noxDataEnv, "texture"):
                row = self.layout.row()
                row.prop(world.noxDataEnv.texture, "enabled")
                row = self.layout.row()
                row.prop(world.noxDataEnv, "azimuth")
                row = self.layout.row()
                row.prop(world.noxDataEnv, "power")
                row = self.layout.row()
                row.prop(world.noxDataEnv, "blendlayer")
                
                
                tslot = world.noxDataEnv.texture
                self.layout.separator()
                row = self.layout.row()
                row.label(text="Choose texture:")
                row = self.layout.row()
                
                col = row.column(align=True)
                #col.template_list(bpy.data, "textures", tslot, "texID")
                col.template_list("UI_UL_list", "wtf3", bpy.data, "textures", tslot, "texID")
                
                col = row.column(align=True)
                col.operator("material.add_nox_image_texture")
                
               
                if tslot.texID<0:
                    col.label(text="No texture chosen")
                    #pass
                else:
                    col.operator("world.uncheck_tex_in_nox_env")
                    tex = bpy.data.textures[tslot.texID]
                    if hasattr(tex, "image_user") and tex.type=="IMAGE":
                        row1 = self.layout.row()
                        col1 = row1.column()
                        
                        if hasattr(tex.image, "filepath"):
                            col1.template_image(tex, "image", tex.image_user, compact=True)
                        else:
                            col1.template_image(tex, "image", tex.image_user, compact=False)
                            

                        row1 = self.layout.row()
                        row1.template_preview(id=bpy.data.textures[tslot.texID], show_buttons=False)
                    else:
                        col.label(text="Not an image texture")
                
                
                row = self.layout.row()
                row.label(text="Modifiers:")
                                
                row = self.layout.row()
                col = row.column(align=True)
                col.prop(tslot, "brightness")
                col.prop(tslot, "contrast")
                col.prop(tslot, "saturation")
                col.prop(tslot, "hue")
                col = row.column(align=True)
                col.prop(tslot, "red")
                col.prop(tslot, "green")
                col.prop(tslot, "blue")
                col.prop(tslot, "invert")
                
                
    @classmethod
    def poll(cls, context):
        return (context.scene.render.engine == 'nox_renderer')

#-----------------------------------
