import bpy
from mathutils import Color
from xml.dom import minidom
import struct
import random
#from io_utils import ImportHelper
from bpy_extras.io_utils import ImportHelper
from bpy.props import StringProperty, BoolProperty, EnumProperty
import os

nox_xml_error_line = -1
nox_xml_error_node = ""
material_folder = ""

#----------------------------------------------------------------

def import_material(filename, mnum):
    global nox_xml_error_node
    global material_folder
    if mnum<0 or mnum >= bpy.data.materials.__len__():
        return False
    
    mat = bpy.data.materials[mnum]
    if not hasattr(mat, "noxData"):
        return False
        
    material_folder = os.path.dirname(filename)

    xmlTree = minidom.parse(filename)

    rootNodes = xmlTree.childNodes
    if rootNodes.__len__() != 1:
        return False
    
    xmlRoot = rootNodes[0]
    if xmlRoot.nodeName!="material":
        nox_xml_error_node = xmlBlend.nodeName
        return False
    
    # <blendlayer>
    for xmlBlend in xmlRoot.getElementsByTagName("blendlayer"):
        res = parseForInt(xmlBlend)
        if not res[0]:
            return False
        if res[1]<1 or res[1]>16:
            nox_xml_error_node = xmlBlend.nodeName
            return False
        mat.noxData.blend_layer = "b"+str(res[1])
        
    # <skyportal> 
    for xmlSkyportal in xmlRoot.getElementsByTagName("skyportal"):
        if not xmlSkyportal.childNodes:
            nox_xml_error_node = xmlSkyportal.nodeName
            return False
        if xmlSkyportal.childNodes[0].nodeType != xmlSkyportal.TEXT_NODE:
            nox_xml_error_node = xmlSkyportal.nodeName
            return False
        if xmlSkyportal.childNodes[0].data == "yes":
            mat.noxData.skyportal = True
        else:
            mat.noxData.skyportal = False

    # <opacity>
    for xmlOpac in xmlRoot.getElementsByTagName("opacity"):
        res = parseForFloat(xmlOpac)
        if not res[0]:
            return False
        if res[1]<0.0 or res[1]>100.0:
            nox_xml_error_node = xmlOpac.nodeName
            return False
        mat.noxData.opacity = res[1]
        
    # <displacement_depth>
    for xmlDisplDepth in xmlRoot.getElementsByTagName("displacement_depth"):
        res = parseForFloat(xmlDisplDepth)
        if not res[0]:
            return False
        if res[1]<0.0 or res[1]>10.0:
            nox_xml_error_node = xmlDisplDepth.nodeName
            return False
        mat.noxData.displ_depth = res[1]
        
    # <displacement_subdivs>
    for xmlDisplSubdivs in xmlRoot.getElementsByTagName("displacement_subdivs"):
        res = parseForInt(xmlDisplSubdivs)
        if not res[0]:
            return False
        if res[1]<1 or res[1]>8:
            nox_xml_error_node = xmlDisplSubdivs.nodeName
            return False
        mat.noxData.displ_subdivs = str(res[1])
        
    # <displacement_shift>
    for xmlDisplShift in xmlRoot.getElementsByTagName("displacement_shift"):
        res = parseForFloat(xmlDisplShift)
        if not res[0]:
            return False
        if res[1]<-10.0 or res[1]>10.0:
            nox_xml_error_node = xmlDisplShift.nodeName
            return False
        mat.noxData.displ_shift = res[1]
        
    # <displacement_continuity> 
    for xmlDisplCont in xmlRoot.getElementsByTagName("displacement_continuity"):
        if not xmlDisplCont.childNodes:
            nox_xml_error_node = xmlDisplCont.nodeName
            return False
        if xmlDisplCont.childNodes[0].nodeType != xmlDisplCont.TEXT_NODE:
            nox_xml_error_node = xmlDisplCont.nodeName
            return False
        if xmlDisplCont.childNodes[0].data == "yes":
            mat.noxData.displ_continuity = True
        else:
            mat.noxData.displ_continuity = False
        
    # <displacement_ignore_scale> 
    for xmlDisplCont in xmlRoot.getElementsByTagName("displacement_ignore_scale"):
        if not xmlDisplCont.childNodes:
            nox_xml_error_node = xmlDisplCont.nodeName
            return False
        if xmlDisplCont.childNodes[0].nodeType != xmlDisplCont.TEXT_NODE:
            nox_xml_error_node = xmlDisplCont.nodeName
            return False
        if xmlDisplCont.childNodes[0].data == "yes":
            mat.noxData.displ_ignore_scale = True
        else:
            mat.noxData.displ_ignore_scale = False
        
    # <displacement_normal_mode>
    for xmlDisplNormal in xmlRoot.getElementsByTagName("displacement_normal_mode"):
        res = parseForInt(xmlDisplNormal)
        if not res[0]:
            return False
        if res[1]<0 or res[1]>2:
            nox_xml_error_node = xmlDisplNormal.nodeName
            return False
        mat.noxData.displ_normal_mode = str(res[1])

    # del old layers
    ll = mat.noxData.layers.__len__()
    for i in range(ll):
        mat.noxData.layers.remove(ll-i-1)

    # <layer>
    for xmlmatlayer in xmlRoot.getElementsByTagName("layer"):
        res = parseMatLayer(xmlmatlayer, mat)
        if not res:
            return False

    # <tex_opacity>
    for xmlTex in xmlRoot.getElementsByTagName("tex_opacity"):
        res = parseTexture(xmlTex, mat, 0)
        if not res:
            return False
            
    # <tex_displacement>
    for xmlTex in xmlRoot.getElementsByTagName("tex_displacement"):
        res = parseTexture(xmlTex, mat, 0)
        if not res:
            return False

            
    # <preview>
    for xmlpreview in xmlRoot.getElementsByTagName("preview"):
        res = parsePreview(xmlpreview, mat)
        if not res:
            return False
    
    return True

#----------------------------------------------------------------

def parseTexture(xmlTex, mat, cLay):
    global nox_xml_error_node
    if xmlTex.nodeName=='tex_opacity':
        tslot = mat.noxData.tex_opacity
    else:
        if xmlTex.nodeName=='tex_displacement':
            tslot = mat.noxData.tex_displacement
        else:
            if cLay>= mat.noxData.layers.__len__() or cLay<0:
                nox_xml_error_node = xmlTex.nodeName
                return False
            layer = mat.noxData.layers[cLay]
            if xmlTex.nodeName=='tex_refl0':
                tslot = layer.tex_refl0
            if xmlTex.nodeName=='tex_refl90':
                tslot = layer.tex_refl90
            if xmlTex.nodeName=='tex_rough':
                tslot = layer.tex_rough
            if xmlTex.nodeName=='tex_weight':
                tslot = layer.tex_weight
            if xmlTex.nodeName=='tex_light':
                tslot = layer.tex_light
            if xmlTex.nodeName=='tex_normal':
                tslot = layer.tex_normal
            if xmlTex.nodeName=='tex_transm':
                tslot = layer.tex_transm
            if xmlTex.nodeName=='tex_aniso':
                tslot = layer.tex_aniso
            if xmlTex.nodeName=='tex_aniso_angle':
                tslot = layer.tex_aniso_angle

    if tslot is None:
        print("unknown texture slot: "+xmlTex.nodeName)
        return False
        
    print("parse texture for slot "+xmlTex.nodeName)
    
    attr_enabled = xmlTex.getAttribute("enabled")
    if attr_enabled == "yes":
        tslot.enabled = True
    else:
        tslot.enabled = False
            
    # <filename>
    for xmlFilename in xmlTex.getElementsByTagName("filename"):
        if not xmlFilename.childNodes:
            nox_xml_error_node = xmlFilename.nodeName
            return (False, 0)
        if xmlFilename.childNodes[0].nodeType != xmlFilename.TEXT_NODE:
            nox_xml_error_node = xmlFilename.nodeName
            return (False, 0)
        if xmlFilename.childNodes[0].data != "":
            res = findOrLoadTexture(xmlFilename.childNodes[0].data)
            if res>=0:
                tslot.texID = res
                tslot.tex_name = bpy.data.textures[res].name
                bpy.data.textures[res].use_fake_user = True

    # <post>
    for xmlPost in xmlTex.getElementsByTagName("post"):

        # <brightness>
        for xmlBrightness in xmlPost.getElementsByTagName("brightness"):
            res = parseForFloat(xmlBrightness)
            if not res[0]:
                return False
            if res[1]<-1.0 or res[1]>1.0:
                nox_xml_error_node = xmlBrightness.nodeName
                return False
            tslot.brightness = res[1]
    
        # <contrast>
        for xmlContrast in xmlPost.getElementsByTagName("contrast"):
            res = parseForFloat(xmlContrast)
            if not res[0]:
                return False
            if res[1]<0.0 or res[1]>4.0:
                nox_xml_error_node = xmlContrast.nodeName
                return False
            tslot.contrast = res[1]

        # <saturation>
        for xmlSaturation in xmlPost.getElementsByTagName("saturation"):
            res = parseForFloat(xmlSaturation)
            if not res[0]:
                return False
            if res[1]<-1.0 or res[1]>1.0:
                nox_xml_error_node = xmlSaturation.nodeName
                return False
            tslot.saturation = res[1]

        # <hue>
        for xmlHue in xmlPost.getElementsByTagName("hue"):
            res = parseForFloat(xmlHue)
            if not res[0]:
                return False
            if res[1]<0.0 or res[1]>4.0:
                nox_xml_error_node = xmlHue.nodeName
                return False
            tslot.hue = res[1]

        # <red>
        for xmlRed in xmlPost.getElementsByTagName("red"):
            res = parseForFloat(xmlRed)
            if not res[0]:
                return False
            if res[1]<-1.0 or res[1]>1.0:
                nox_xml_error_node = xmlRed.nodeName
                return False
            tslot.red = res[1]

        # <green>
        for xmlGreen in xmlPost.getElementsByTagName("green"):
            res = parseForFloat(xmlGreen)
            if not res[0]:
                return False
            if res[1]<-1.0 or res[1]>1.0:
                nox_xml_error_node = xmlGreen.nodeName
                return False
            tslot.green = res[1]

        # <blue>
        for xmlBlue in xmlPost.getElementsByTagName("blue"):
            res = parseForFloat(xmlBlue)
            if not res[0]:
                return False
            if res[1]<-1.0 or res[1]>1.0:
                nox_xml_error_node = xmlBlue.nodeName
                return False
            tslot.blue = res[1]

        # <gamma>
        for xmlGamma in xmlPost.getElementsByTagName("gamma"):
            res = parseForFloat(xmlGamma)
            if not res[0]:
                return False
            if res[1]<1.0 or res[1]>4.0:
                nox_xml_error_node = xmlGamma.nodeName
                return False
            tslot.gamma = res[1]

        # <invert>
        for xmlInvert in xmlPost.getElementsByTagName("invert"):
            if not xmlInvert.childNodes:
                nox_xml_error_node = xmlInvert.nodeName
                return (False, 0)
            if xmlInvert.childNodes[0].nodeType != xmlInvert.TEXT_NODE:
                nox_xml_error_node = xmlInvert.nodeName
                return (False, 0)
            if xmlInvert.childNodes[0].data == "yes":
                tslot.invert = True
            else:
                tslot.invert = False
                
        # <filter>
        for xmlFilter in xmlPost.getElementsByTagName("filter"):
            res = parseForInt(xmlFilter)
            if not res[0]:
                return False
            if res[1]==1:
                tslot.bilinear= True
            else:
                tslot.bilinear = False
                
    # <post_normal>
    for xmlPostNormal in xmlTex.getElementsByTagName("post_normal"):

        # <pn_power>
        for xmlPower in xmlPostNormal.getElementsByTagName("pn_power"):
            res = parseForFloat(xmlPower)
            if not res[0]:
                return False
            if res[1]<-5.0 or res[1]>5.0:
                nox_xml_error_node = xmlPower.nodeName
                return False
            tslot.normal_power = res[1]
    
        # <pn_invert_x>
        for xmlInvertX in xmlPostNormal.getElementsByTagName("pn_invert_x"):
            if not xmlInvertX.childNodes:
                nox_xml_error_node = xmlInvertX.nodeName
                return (False, 0)
            if xmlInvertX.childNodes[0].nodeType != xmlInvertX.TEXT_NODE:
                nox_xml_error_node = xmlInvertX.nodeName
                return (False, 0)
            if xmlInvertX.childNodes[0].data == "yes":
                tslot.normal_invert_x = True
            else:
                tslot.normal_invert_x = False
                
        # <pn_invert_y>
        for xmlInvertY in xmlPostNormal.getElementsByTagName("pn_invert_y"):
            if not xmlInvertY.childNodes:
                nox_xml_error_node = xmlInvertY.nodeName
                return (False, 0)
            if xmlInvertY.childNodes[0].nodeType != xmlInvertY.TEXT_NODE:
                nox_xml_error_node = xmlInvertY.nodeName
                return (False, 0)
            if xmlInvertY.childNodes[0].data == "yes":
                tslot.normal_invert_y = True
            else:
                tslot.normal_invert_y = False
                
    return True
    
#----------------------------------------------------------------

def parseMatLayer(xmlLayer, mat):
    global nox_xml_error_node
    
    print("parseMatLayer")
    
    if mat.noxData.layers.__len__()>15:
        nox_xml_error_node = xmlLayer.nodeName + " too many!"
        return False
    
    mat.noxData.layers.add()
    cLay = mat.noxData.layers.__len__()-1
    mlay = mat.noxData.layers[cLay]
    mlay.name = "Layer " + str(mat.noxData.layers.__len__())
    
    # <layer type>
    ltype = xmlLayer.getAttribute("type")
    if ltype=="shade":
        mlay.emission = False
    elif ltype=="emission":
        mlay.emission = True
    else:
        nox_xml_error_node = xmlLayer.nodeName
        return False
        
    # <layer contribution>
    scontr = xmlLayer.getAttribute("contribution")
    try:
        v = float(scontr)
    except ValueError:
        nox_xml_error_node = xmlLayer.nodeName
        return False
    mlay.weight = v
    
    # <diffuse>
    for xmlDiff in xmlLayer.getElementsByTagName("diffuse"):
        res = parseForColorHex(xmlDiff)
        if not res[0]:
            return False
        mlay.refl0.r = float(res[1])/255.0
        mlay.refl0.g = float(res[2])/255.0
        mlay.refl0.b = float(res[3])/255.0
    
    # <diffuse90>
    for xmlDiff in xmlLayer.getElementsByTagName("diffuse90"):
        res = parseForColorHex(xmlDiff)
        if not res[0]:
            return False
        mlay.refl90.r = float(res[1])/255.0
        mlay.refl90.g = float(res[2])/255.0
        mlay.refl90.b = float(res[3])/255.0

    # <connect90 active>
    for xmlConnect90 in xmlLayer.getElementsByTagName("connect90"):
        c90act = xmlConnect90.getAttribute("active")
        if c90act=="yes":
            mlay.refl_linked = True
        else:
            mlay.refl_linked = False
        
    # <roughness>
    for xmlRough in xmlLayer.getElementsByTagName("roughness"):
        res = parseForFloat(xmlRough)
        if not res[0]:
            return False
        if res[1]<0.0 or res[1]>100.0:
            nox_xml_error_node = xmlRough.nodeName
            return False
        mlay.rough = res[1]

    # <anisotropy>
    for xmlAniso in xmlLayer.getElementsByTagName("anisotropy"):
        res = parseForFloat(xmlAniso)
        if not res[0]:
            return False
        if res[1]<0.0 or res[1]>100.0:
            nox_xml_error_node = xmlAniso.nodeName
            return False
        mlay.anisotropy = res[1]
        
    # <anisotropy_angle>
    for xmlAnisoAngle in xmlLayer.getElementsByTagName("anisotropy_angle"):
        res = parseForFloat(xmlAnisoAngle)
        if not res[0]:
            return False
        if res[1]<0.0 or res[1]>100.0:
            nox_xml_error_node = xmlAnisoAngle.nodeName
            return False
        mlay.aniso_angle = res[1]
        
    # <fresnel>
    for xmlFres in xmlLayer.getElementsByTagName("fresnel"):
        # <ior>
        for xmlIOR in xmlFres.getElementsByTagName("ior"):
            res = parseForFloat(xmlIOR)
            if not res[0]:
                return False
            if res[1]<1.0 or res[1]>100.0:
                nox_xml_error_node = xmlIOR.nodeName
                return False
            mlay.ior = res[1]
            
    # print("5")

    # <transmission>
    for xmlTransm in xmlLayer.getElementsByTagName("transmission"):
        # <transmission active>
        tract = xmlTransm.getAttribute("active")
        if tract=="yes":
            mlay.transmission = True
        else:
            mlay.transmission = False
           
        # <transmcolor>
        for xmltrcol in xmlTransm.getElementsByTagName("transmcolor"):
            res = parseForColorHex(xmltrcol)
            if not res[0]:
                return False
            mlay.transm_col.r = float(res[1])/255.0
            mlay.transm_col.g = float(res[2])/255.0
            mlay.transm_col.b = float(res[3])/255.0
        
        # <fakeglass> 
        for xmlFakeglass in xmlTransm.getElementsByTagName("fakeglass"):
            if not xmlFakeglass.childNodes:
                nox_xml_error_node = xmlFakeglass.nodeName
                return (False, 0)
            if xmlFakeglass.childNodes[0].nodeType != xmlFakeglass.TEXT_NODE:
                nox_xml_error_node = xmlFakeglass.nodeName
                return (False, 0)
            if xmlFakeglass.childNodes[0].data == "yes":
                mlay.fake_glass = True
            else:
                mlay.fake_glass = False

        #<absorption>        
        for xmlAbsorpt in xmlTransm.getElementsByTagName("absorption"):
            # <absorption active>
            tract = xmlAbsorpt.getAttribute("active")
            if tract=="yes":
                mlay.absorption = True
            else:
                mlay.absorption = False
    
            # <abscolor>
            for xmlAbsCol in xmlAbsorpt.getElementsByTagName("abscolor"):
                res = parseForColorHex(xmlAbsCol)
                if not res[0]:
                    return False
                mlay.absorption_color.r = float(res[1])/255.0
                mlay.absorption_color.g = float(res[2])/255.0
                mlay.absorption_color.b = float(res[3])/255.0
            
            # <distance>
            for xmlAbsDist in xmlAbsorpt.getElementsByTagName("distance"):
                res = parseForFloat(xmlAbsDist)
                if not res[0]:
                    return False
                if res[1]<0.0 or res[1]>100.0:
                    nox_xml_error_node = xmlAbsDist.nodeName
                    return False
                mlay.absorption_distance = res[1]

    # <sss>
    for xmlSSS in xmlLayer.getElementsByTagName("sss"):
        # <sss active>
        sssact = xmlSSS.getAttribute("active")
        if sssact=="yes":
            mlay.sss_on = True
        else:
            mlay.sss_on = False
            
        # <sss_density>
        for xmlSSSdens in xmlSSS.getElementsByTagName("sss_density"):
            res = parseForFloat(xmlSSSdens)
            if not res[0]:
                return False
            if res[1]<0.0 or res[1]>1000.0:
                nox_xml_error_node = xmlSSSdens.nodeName
                return False
            mlay.sss_density = res[1]

        # <sss_coll_dir>
        for xmlSSScdir in xmlSSS.getElementsByTagName("sss_coll_dir"):
            res = parseForFloat(xmlSSScdir)
            if not res[0]:
                return False
            if res[1]<-1.0 or res[1]>1.0:
                nox_xml_error_node = xmlSSScdir.nodeName
                return False
            mlay.sss_coll_dir = res[1]

    # <emission>
    for xmlEmissCol in xmlLayer.getElementsByTagName("emission"):
        res = parseForColorHex(xmlEmissCol)
        if not res[0]:
            return False
        mlay.emission_color.r = float(res[1])/255.0
        mlay.emission_color.g = float(res[2])/255.0
        mlay.emission_color.b = float(res[3])/255.0
    
    # <power>
    for xmlEmissPow in xmlLayer.getElementsByTagName("power"):
        res = parseForFloatPackedHex(xmlEmissPow)
        if not res[0]:
            return False
        if res[1]<0.0:
            nox_xml_error_node = xmlEmissPow.nodeName
            return False
        mlay.emission_power = res[1]

    # <unit>
    for xmlUnit in xmlLayer.getElementsByTagName("unit"):
        if not xmlUnit.childNodes:
            nox_xml_error_node = xmlUnit.nodeName
            return (False, 0)
        if xmlUnit.childNodes[0].nodeType != xmlUnit.TEXT_NODE:
            nox_xml_error_node = xmlUnit.nodeName
            return (False, 0)
        ustr = xmlUnit.childNodes[0].data
        mlay.emission_unit = 'W'
        if ustr=="w":
            mlay.emission_unit = 'W'
        if ustr=="w/m2":
            mlay.emission_unit = 'Wm'
        if ustr=="lm":
            mlay.emission_unit = 'lm'
        if ustr=="lx":
            mlay.emission_unit = 'lx'
        
    # <temperature>
    for xmlTemperature in xmlLayer.getElementsByTagName("temperature"):
        res = parseForInt(xmlTemperature)
        if not res[0]:
            return False
        if res[1]<1000 or res[1]>10000:
            nox_xml_error_node = xmlTemperature.nodeName
            return False
        mlay.emission_temperature = res[1]

    # <ies>
    for xmlIES in xmlLayer.getElementsByTagName("ies"):
        if not xmlIES.childNodes:
            nox_xml_error_node = xmlIES.nodeName
            return (False, 0)
        if xmlIES.childNodes[0].nodeType != xmlIES.TEXT_NODE:
            nox_xml_error_node = xmlIES.nodeName
            return (False, 0)
        if xmlIES.childNodes[0].data != "":
            mlay.ies_file = xmlIES.childNodes[0].data
            
    # <tex_refl0>
    for xmlTex in xmlLayer.getElementsByTagName("tex_refl0"):
        res = parseTexture(xmlTex, mat, cLay)
        if not res:
            return False

    # <tex_refl90>
    for xmlTex in xmlLayer.getElementsByTagName("tex_refl90"):
        res = parseTexture(xmlTex, mat, cLay)
        if not res:
            return False

    # <tex_rough>
    for xmlTex in xmlLayer.getElementsByTagName("tex_rough"):
        res = parseTexture(xmlTex, mat, cLay)
        if not res:
            return False

    # <tex_weight>
    for xmlTex in xmlLayer.getElementsByTagName("tex_weight"):
        res = parseTexture(xmlTex, mat, cLay)
        if not res:
            return False

    # <tex_light>
    for xmlTex in xmlLayer.getElementsByTagName("tex_light"):
        res = parseTexture(xmlTex, mat, cLay)
        if not res:
            return False
            
    # <tex_normal>
    for xmlTex in xmlLayer.getElementsByTagName("tex_normal"):
        res = parseTexture(xmlTex, mat, cLay)
        if not res:
            return False

    # <tex_transm>
    for xmlTex in xmlLayer.getElementsByTagName("tex_transm"):
        res = parseTexture(xmlTex, mat, cLay)
        if not res:
            return False
            
    # <tex_aniso>
    for xmlTex in xmlLayer.getElementsByTagName("tex_aniso"):
        res = parseTexture(xmlTex, mat, cLay)
        if not res:
            return False

    # <tex_aniso_angle>
    for xmlTex in xmlLayer.getElementsByTagName("tex_aniso_angle"):
        res = parseTexture(xmlTex, mat, cLay)
        if not res:
            return False

    return True

#----------------------------------------------------------------
    
def parseForInt(xmlNode):
    global nox_xml_error_node
    if not xmlNode.childNodes:
        nox_xml_error_node = xmlNode.nodeName
        return (False, 0)
    if xmlNode.childNodes[0].nodeType != xmlNode.TEXT_NODE:
        nox_xml_error_node = xmlNode.nodeName
        return (False, 0)
    try:
        v = int(xmlNode.childNodes[0].data)
    except ValueError:
        nox_xml_error_node = xmlNode.nodeName
        return (False, 0)
    return (True, v)

#----------------------------------------------------------------
    
def parseForFloat(xmlNode):
    global nox_xml_error_node
    if not xmlNode.childNodes:
        nox_xml_error_node = xmlNode.nodeName
        return (False, 0.0)
    if xmlNode.childNodes[0].nodeType != xmlNode.TEXT_NODE:
        nox_xml_error_node = xmlNode.nodeName
        return (False, 0.0)
    try:
        v = float(xmlNode.childNodes[0].data)
    except ValueError:
        nox_xml_error_node = xmlNode.nodeName
        return (False, 0.0)
    return (True, v)

#----------------------------------------------------------------

def parseForColorHex(xmlNode):
    global nox_xml_error_node
    if not xmlNode.childNodes:
        nox_xml_error_node = xmlNode.nodeName
        return (False, 0,0,0)
    if xmlNode.childNodes[0].nodeType != xmlNode.TEXT_NODE:
        nox_xml_error_node = xmlNode.nodeName
        return (False, 0,0,0)
    hexstr = xmlNode.childNodes[0].data
    if len(hexstr) != 6:
        nox_xml_error_node = xmlNode.nodeName
        return (False, 0,0,0)
    hr = "{0}{1}".format(*hexstr)
    hg = "{2}{3}".format(*hexstr)
    hb = "{4}{5}".format(*hexstr)
    try:
        dr = int(hr, 16)
        dg = int(hg, 16)
        db = int(hb, 16)
    except ValueError:
        nox_xml_error_node = xmlNode.nodeName
        return (False, 0,0,0)
    
    return (True, dr,dg,db)

#----------------------------------------------------------------

def parseForFloatPackedHex(xmlNode):
    global nox_xml_error_node
    if not xmlNode.childNodes:
        nox_xml_error_node = xmlNode.nodeName
        return (False, 0.0)
    if xmlNode.childNodes[0].nodeType != xmlNode.TEXT_NODE:
        nox_xml_error_node = xmlNode.nodeName
        return (False, 0.0)
    hexstr = xmlNode.childNodes[0].data
    if len(hexstr) != 8:
        nox_xml_error_node = xmlNode.nodeName
        return (False, 0.0)
    h1 = "{1}{0}".format(*hexstr)
    h2 = "{3}{2}".format(*hexstr)
    h3 = "{5}{4}".format(*hexstr)
    h4 = "{7}{6}".format(*hexstr)
    try:
        d1 = int(h1, 16)
        d2 = int(h2, 16)
        d3 = int(h3, 16)
        d4 = int(h4, 16)
    except ValueError:
        nox_xml_error_node = xmlNode.nodeName
        return (False, 0.0)
    
    aastr = struct.pack("=BBBB", d1,d2,d3,d4)
    f = struct.unpack("=f", aastr)
    
    return (True, f[0])

#----------------------------------------------------------------

def parsePreview(xmlPreview, mat):
    global nox_xml_error_node
    
    #return True
    print("parsing preview")
    strwidth  = xmlPreview.getAttribute("width")
    strheight = xmlPreview.getAttribute("height")
    print("width = "+strwidth)
    print("height = "+strheight)
    try:
        iw = int(strwidth)
        ih = int(strheight)
    except ValueError:
        nox_xml_error_node = xmlPreview.nodeName
        return False
    
    #if mat.noxData.preview_id<0 or mat.noxData.preview_id>=bpy.data.images.__len__():
    #    bpy.ops.image.new(width=iw, height=ih)
    #    mat.noxData.preview_id = bpy.data.images.__len__()-1
    #img = bpy.data.images[mat.noxData.preview_id]
    #bpy.ops.texture.new()
    #tid = bpy.data.textures.__len__()-1
    #bpy.data.textures[tid].type="IMAGE"
    #bpy.data.textures[tid].type_recast


    #if mat.noxData.preview_id<0 or mat.noxData.preview_id>=bpy.data.images.__len__():
    #    bpy.ops.image.new(width=iw, height=ih)
    #    mat.noxData.preview_id = bpy.data.images.__len__()-1
    #img = bpy.data.images[mat.noxData.preview_id]
    
    imagename = "prev_mat_"+mat.name
    if not imagename in bpy.data.images:
        print("creating new image")
        bpy.ops.image.new(name=imagename, width=iw, height=ih)
        mat.noxData.preview_id = bpy.data.images.__len__()-1
        mat.noxData.preview_image_name = "prev_mat_"+mat.name
        print ("added new image")
    img = bpy.data.images[imagename]
    img.use_fake_user = True
    
    print("parsing data")
        
    arr2 = [None] * iw * ih
    #for i in range(ih):
    for i in range(ih-1, -1, -1):
        nodestr = "l"+str(i)
        # <lxxx>
        lstart = iw*4*i
        for xmlLine in xmlPreview.getElementsByTagName(nodestr):
            if not xmlLine.childNodes:
                nox_xml_error_node = xmlLine.nodeName
                return False
            if xmlLine.childNodes[0].nodeType != xmlLine.TEXT_NODE:
                nox_xml_error_node = xmlLine.nodeName
                return False
            ustr = xmlLine.childNodes[0].data
            #print(nodestr)
            if len(ustr) != iw * 6:
                nox_xml_error_node = xmlLine.nodeName
                return False
            for j in range(iw):
                dst = lstart + j*4
                src = j*6
                
                hr =  ustr[src+0]+ustr[src+1]
                hg =  ustr[src+2]+ustr[src+3]
                hb =  ustr[src+4]+ustr[src+5]
                
                try:
                    dr = int(hr, 16)
                    dg = int(hg, 16)
                    db = int(hb, 16)
                except ValueError:
                    nox_xml_error_node = xmlNode.nodeName
                    return False
                fr = float(dr)/255.0
                fg = float(dg)/255.0
                fb = float(db)/255.0
                #code below is extremely slow
                #img.pixels[dst+0] = fr
                #img.pixels[dst+1] = fg
                #img.pixels[dst+2] = fb
                #img.pixels[dst+3] = 1.0
                k = ih-i-1
                arr2[k*iw+j] = [fr, fg, fb, 1.0]

    arr2 = [chan for px in arr2 for chan in px]
    img.pixels = arr2
    
    img.update()
    img.pack(as_png=True)
    
    print("preview loaded")
    
    return True    

#----------------------------------------------------------------

def findOrLoadTexture(filename):
    global material_folder
    res = -1
    print("trying texture "+filename)
    if not os.path.isabs(filename):
        filename = os.path.join(material_folder, filename)
        print("changed to : "+filename)        
    filename = os.path.abspath(filename)
    for i in range(bpy.data.textures.__len__()):
        tex = bpy.data.textures[i]   
        if hasattr(tex, "image") and tex.type=="IMAGE":
            if hasattr(tex.image, 'filepath') and hasattr(tex.image, 'source'):
                if tex.image.source=='FILE' and os.path.abspath(bpy.path.abspath(tex.image.filepath))==filename:
                    print("already loaded id: "+str(i))
                    res = i
    if res==-1:
        print("loading image")
        #print("path absolute: "+str(os.path.isabs(filename)))
        try:
            bpy.ops.image.open(filepath=filename)
        except RuntimeError:
            return -1

        #print("looking for "+filename)
        imgid = -1
        for i in range(bpy.data.images.__len__()):
            timg = bpy.data.images[i]            
            if timg.type=='IMAGE' and os.path.abspath(bpy.path.abspath(timg.filepath))==filename:
                imgid = i
            #print(timg.filepath)
            #print(os.path.isabs(timg.filepath))
            #print(bpy.path.abspath(timg.filepath))
            #print(os.path.abspath(bpy.path.abspath(timg.filepath)))

        if imgid<0:
            print("Internal error, loaded texture but can't find it. Funny, I know...")
            return -1
            
        #imgid = bpy.data.images.__len__()-1
        img = bpy.data.images[imgid]

        print("loaded img id: "+str(imgid))

        bpy.ops.texture.new()
        texID = bpy.data.textures.__len__()-1
        tex = bpy.data.textures[texID]
        tex.type = 'IMAGE'
        tex.type_recast()
        tex = bpy.data.textures[texID]
        tex.image = img
        res = texID

        print("and tex id: "+str(texID))
        
    return res
    
#----------------------------------------------------------------

class NOXMaterialImporter(bpy.types.Operator, ImportHelper):
    bl_idname = "wm.buttonimportnoxmaterial"
    bl_label = "Import NOX Material"
    filename_ext = ".nxm"
    filter_glob = StringProperty(default="*.nxm", options={'HIDDEN'})
    	
    #addTex = BoolProperty(name="Copy textures", 
    #    description="Copy used textures - not implemented", default=False)
       
    def invoke(self, context, event):
        wm = context.window_manager
        wm.fileselect_add(self)
        return {"RUNNING_MODAL"}
        
    def execute(self, context):
        matID = -1
        matname = ""
        if hasattr(context, "material"):
            if hasattr(context.material, "name"):
                matname = context.material.name
                nummats = bpy.data.materials.__len__()
                for i in range(nummats):
                    if matname == bpy.data.materials[i].name:
                        matID = i
						
        if matID > -1:
            i_ok = import_material(self.filepath, matID)
            if not i_ok:
                print("nox material import error with <" + nox_xml_error_node + ">")
            material = bpy.data.materials[matID]
            if hasattr(material, "noxData"):
                if material.noxData.layers.__len__()>0:
                    if material.noxData.layers[0].emission:
                        material.diffuse_color = material.noxData.layers[0].emission_color
                        material.specular_color = (0.0, 0.0, 0.0)
                    else:
                        material.diffuse_color = material.noxData.layers[0].refl0
                        material.specular_color = material.noxData.layers[0].refl90
        return {"FINISHED"}
    
def menu_func_import(self, context):
    self.layout.operator(NOXMaterialImporter.bl_idname, text="NOX Material Import (*.nxm)")

#----------------------------------------------------------------
    
