import bpy
import tempfile
import platform

#-----------------------------------

class RunMatEditorOperator(bpy.types.Operator):
    bl_idname = "wm.buttonrunnoxmateditor"
    bl_label = "Run NOX Material Editor"
    def execute(self, context):
        td = tempfile.TemporaryDirectory()
        noxmatfile = td.name+'\\tempfile.nxm'
        bpy.ops.wm.buttonexportnoxmaterial(filepath=noxmatfile)
        runMatEditor(noxmatfile)
        bpy.ops.wm.buttonimportnoxmaterial(filepath=noxmatfile)
        return{'FINISHED'}
    
#-----------------------------------
    
class add_nox_image_texture(bpy.types.Operator):
    bl_label = "Add Image Texture"
    bl_idname = "material.add_nox_image_texture"
    
    def invoke(self, context, event):
        bpy.ops.texture.new()
        tid = bpy.data.textures.__len__()-1
        bpy.data.textures[tid].type="IMAGE"
        bpy.data.textures[tid].type_recast
        bpy.data.textures[tid].use_fake_user = True
        return {'FINISHED'}


#-----------------------------------
    
class change_tex_type_to_image(bpy.types.Operator):
    bl_label = "Change to image tex"
    bl_idname = "material.change_tex_type_to_image"
    
    def invoke(self, context, event):
        mat = context.material
        if mat.noxData.layers.__len__()<1 or mat.noxData.layers.__len__()<=mat.noxData.lay_index:
            return {'FINISHED'}
        layer = mat.noxData.layers[mat.noxData.lay_index]
        
        if layer.textures=="refl0":
            tslot = layer.tex_refl0
        if layer.textures=="refl90":
            tslot = layer.tex_refl90
        if layer.textures=="rough":
            tslot = layer.tex_rough
        if layer.textures=="light":
            tslot = layer.tex_light
        if layer.textures=="weight":
            tslot = layer.tex_weight
        if layer.textures=="normal":
            tslot = layer.tex_normal
        if layer.textures=="transm":
            tslot = layer.tex_transm
        if layer.textures=="anisotropy":
            tslot = layer.tex_aniso
        if layer.textures=="aniso_angle":
            tslot = layer.tex_aniso_angle
        if layer.textures=="opacity":
            tslot = mat.noxData.tex_opacity
        if layer.textures=="displacement":
            tslot = mat.noxData.tex_displacement
        tid = tslot.texID

        if tid>=bpy.data.textures.__len__():
            return {'FINISHED'}

        bpy.data.textures[tid].type="IMAGE"
        bpy.data.textures[tid].type_recast
        bpy.data.textures[tid].use_fake_user = True
        return {'FINISHED'}


#-----------------------------------

class uncheck_tex_in_nox_slot(bpy.types.Operator):
    bl_label = "Uncheck texture"
    bl_idname = "material.uncheck_tex_in_nox_slot"
    
    def invoke(self, context, event):
        mat = context.material
        if mat.noxData.layers.__len__()<1 or mat.noxData.layers.__len__()<=mat.noxData.lay_index:
            return {'FINISHED'}
        layer = mat.noxData.layers[mat.noxData.lay_index]
        
        if layer.textures=="refl0":
            tslot = layer.tex_refl0
        if layer.textures=="refl90":
            tslot = layer.tex_refl90
        if layer.textures=="rough":
            tslot = layer.tex_rough
        if layer.textures=="light":
            tslot = layer.tex_light
        if layer.textures=="weight":
            tslot = layer.tex_weight
        if layer.textures=="normal":
            tslot = layer.tex_normal
        if layer.textures=="transm":
            tslot = layer.tex_transm
        if layer.textures=="anisotropy":
            tslot = layer.tex_aniso
        if layer.textures=="aniso_angle":
            tslot = layer.tex_aniso_angle
        if layer.textures=="opacity":
            tslot = mat.noxData.tex_opacity
        if layer.textures=="displacement":
            tslot = mat.noxData.tex_displacement
        tslot.texID = -1
        return {'FINISHED'}

#-----------------------------------

class delete_tex_permanently(bpy.types.Operator):
    bl_label = "Delete texture"
    bl_idname = "material.delete_tex_permanently"
    
    def invoke(self, context, event):
        mat = context.material
        if mat.noxData.layers.__len__()<1 or mat.noxData.layers.__len__()<=mat.noxData.lay_index:
            return {'FINISHED'}
        layer = mat.noxData.layers[mat.noxData.lay_index]
        
        if layer.textures=="refl0":
            tslot = layer.tex_refl0
        if layer.textures=="refl90":
            tslot = layer.tex_refl90
        if layer.textures=="rough":
            tslot = layer.tex_rough
        if layer.textures=="light":
            tslot = layer.tex_light
        if layer.textures=="weight":
            tslot = layer.tex_weight
        if layer.textures=="normal":
            tslot = layer.tex_normal
        if layer.textures=="transm":
            tslot = layer.tex_transm
        if layer.textures=="anisotropy":
            tslot = layer.tex_aniso
        if layer.textures=="aniso_angle":
            tslot = layer.tex_aniso_angle
        if layer.textures=="opacity":
            tslot = mat.noxData.tex_opacity
        if layer.textures=="displacement":
            tslot = mat.noxData.tex_displacement
            
        delID = tslot.texID
        
        num_mats = bpy.data.materials.__len__()
        for i in range(num_mats):
            mat = bpy.data.materials[i]
            if mat.noxData:
                if mat.noxData.tex_opacity.texID == delID:
                    mat.noxData.tex_opacity.texID = -1
                if mat.noxData.tex_opacity.texID > delID:
                    mat.noxData.tex_opacity.texID -= 1
                if mat.noxData.tex_displacement.texID == delID:
                    mat.noxData.tex_displacement.texID = -1
                if mat.noxData.tex_displacement.texID > delID:
                    mat.noxData.tex_displacement.texID -= 1
                num_layers = mat.noxData.layers.__len__()
                for j in range(num_layers):
                    layer = mat.noxData.layers[j]
                    if layer.tex_refl0.texID == delID:
                        layer.tex_refl0.texID = -1;
                    if layer.tex_refl0.texID > delID:
                        layer.tex_refl0.texID -= 1;
                    if layer.tex_refl90.texID == delID:
                        layer.tex_refl90.texID = -1;
                    if layer.tex_refl90.texID > delID:
                        layer.tex_refl90.texID -= 1;
                    if layer.tex_rough.texID == delID:
                        layer.tex_rough.texID = -1;
                    if layer.tex_rough.texID > delID:
                        layer.tex_rough.texID -= 1;
                    if layer.tex_light.texID == delID:
                        layer.tex_light.texID = -1;
                    if layer.tex_light.texID > delID:
                        layer.tex_light.texID -= 1;
                    if layer.tex_weight.texID == delID:
                        layer.tex_weight.texID = -1;
                    if layer.tex_weight.texID > delID:
                        layer.tex_weight.texID -= 1;
                    if layer.tex_normal.texID == delID:
                        layer.tex_normal.texID = -1;
                    if layer.tex_normal.texID > delID:
                        layer.tex_normal.texID -= 1;
                    if layer.tex_transm.texID == delID:
                        layer.tex_transm.texID = -1;
                    if layer.tex_transm.texID > delID:
                        layer.tex_transm.texID -= 1;
                    if layer.tex_aniso.texID == delID:
                        layer.tex_aniso.texID = -1;
                    if layer.tex_aniso.texID > delID:
                        layer.tex_aniso.texID -= 1;
                    if layer.tex_aniso_angle.texID == delID:
                        layer.tex_aniso_angle.texID = -1;
                    if layer.tex_aniso_angle.texID > delID:
                        layer.tex_aniso_angle.texID -= 1;
        bpy.data.textures[delID].user_clear()
        bpy.data.textures.remove(bpy.data.textures[delID])
                        
        return {'FINISHED'}

#-----------------------------------


class add_remove_material_layer(bpy.types.Operator):
    bl_label = "Add or Remove"
    bl_idname = "matlayer.add_remove"
 
    add = bpy.props.BoolProperty(default = True)
 
    def invoke(self, context, event):
        add = self.add
        obj = context.material
        collection = obj.noxData.layers
 
        if add:
            # This add at the end of the collection list
            collection.add()
            num = collection.__len__()
            collection[num-1].name= "Layer "+str(num)
        else:
            # This remove on item in the collection list function of index value
            index = obj.noxData.lay_index
            collection.remove(index)
            num = collection.__len__()
            if index >= num:
                index = num-1
            if index < 0:
                index = 0
            obj.noxData.lay_index = index
           
            for i in range(num):
                collection[i].name= "Layer "+str(i+1)
 
        return {'FINISHED'}

#---------------------------------------------------------------------------------------------------

import os
import winreg
from subprocess import call, Popen

def getNOXpath():
	npath = ""
	try:
		reg = winreg.OpenKey(winreg.HKEY_CURRENT_USER, "Software\Evermotion\Renderer")
		res = winreg.QueryValueEx(reg, "directory")
		if (res[1]!=winreg.REG_SZ):
			ok = False
			return (False, "")
		npath = res[0]
	except WindowsError:
		ok = False
		return (False, "")
	winreg.CloseKey(reg)
	ok = True
	return (True, npath)

#---------------------------------------------------------------------------------------------------

def runMatEditor(matfile):
    noxpath = getNOXpath()
    if (noxpath[0]):
        a = platform.architecture()
        if a[0] == '64bit':
            cmdline='""'+noxpath[1]+'\\bin\\64\\MaterialEditor64.exe" -save "'+matfile+'""'
        else:
            cmdline='""'+noxpath[1]+'\\bin\\32\\MaterialEditor32.exe" -save "'+matfile+'""'
        print(cmdline)
        fs=os.system(cmdline)
    else:
        print("can't find nox")
	
#---------------------------------------------------------------------------------------------------

def runNOX(noxfile, startrendering):
    noxpath = getNOXpath()
    if (noxpath[0]):
        #cmdline=""+noxpath[1]+"\\bin\\64\\Renderer64.exe "+noxfile
        a = platform.architecture()
        if a[0] == '64bit':
            if startrendering:
                cmdline=""+noxpath[1]+"\\bin\\64\\Renderer64.exe -start \""+noxfile+"\""
            else:
                cmdline=""+noxpath[1]+"\\bin\\64\\Renderer64.exe \""+noxfile+"\""
        else:
            if startrendering:
                cmdline=""+noxpath[1]+"\\bin\\32\\Renderer32.exe -start \""+noxfile+"\""
            else:
                cmdline=""+noxpath[1]+"\\bin\\32\\Renderer32.exe \""+noxfile+"\""
        #print(cmdline)
        p = Popen(cmdline + "", shell=False)
    else:
        print("can't find nox")

#---------------------------------------------------------------------------------------------------
