import bpy
from render_nox.material.material_ops import runNOX, getNOXpath
from render_nox.export.export_nox_scene import write_NOX_data
import time
import os
import winreg

#---------------------------------------------------------------------------------------------------

def getNoxUserPath():
    if 'LOCALAPPDATA' not in os.environ:
        print("Can't find Local App Data folder")
        return None
    localdir = os.environ['LOCALAPPDATA']
    if not os.path.exists(localdir):
        return None
    noxuserdir = localdir+'\\Evermotion\\NOX'
    if not os.path.exists(noxuserdir):
        return None
    return noxuserdir

#---------------------------------------------------------------------------------------------------

def getNoxUserPathReg():
	npath = ""
	try:
		reg = winreg.OpenKey(winreg.HKEY_CURRENT_USER, "Software\Evermotion\Renderer")
		res = winreg.QueryValueEx(reg, "userdirectory")
		if (res[1]!=winreg.REG_SZ):
			ok = False
			return None
		npath = res[0]
	except WindowsError:
		ok = False
		return None
	winreg.CloseKey(reg)
	ok = True
	return npath
 
#---------------------------------------------------------------------------------------------------

class RunNOXRendererOperator(bpy.types.Operator):
    bl_idname = "wm.buttonrunnoxrenderer"
    bl_label = "Render in NOX"
    def execute(self, context):
        #noxpath = getNOXpath()
        #if not noxpath[0]:
        #    print("Export error: NOX not found.")
        #    return {'CANCELLED'}
        scene = bpy.context.scene
        if not scene:
            print("Export error: Scene is NULL. (invoked from operator)")
            return {'CANCELLED'}
        timestr = time.strftime("_%Y_%m_%d_%H_%M_%S")
        #noxscenefile = noxpath[1]+'\\exported scenes\\'+scene.name+timestr+".nox"

        noxuserpath = getNoxUserPathReg()
        if noxuserpath is None:
            print("Export error: NOX user folder not found in registry.")
            return {'CANCELLED'}
        if not os.path.exists(noxuserpath):
            print("Export error, following path does not exist: "+noxuserpath)
            return {'CANCELLED'}
        #noxuserpath = getNoxUserPath()
        #if noxuserpath is None:
        #    print("Export error: NOX user folder not found.")
        #    return {'CANCELLED'}
        noxscenefile = noxuserpath +'\\exported scenes\\'+scene.name+timestr+".nox"
        print("exporting to file: "+noxscenefile)
        write_NOX_data(bpy.context, noxscenefile, 'OPT_A')
        print("starting nox")
        runNOX(noxscenefile, True)
        return{'FINISHED'}

#---------------------------------------------------------------------------------------------------
