import bpy

#---------------------------------------------------------------------------------------------------

class NOXRenderPanel(bpy.types.Panel):
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_idname = "noxrenderpanel"
    bl_label = "NOX Renderer"
    bl_context = "render"
    COMPAT_ENGINES = {"nox_renderer"}

    
    def draw(self, context):
        sc = context.scene
        
        row = self.layout.row()
        row.prop(sc.render, "resolution_x", text="Width")
        row = self.layout.row()
        row.prop(sc.render, "resolution_y", text="Height")
        row = self.layout.row()
        row.prop(sc.noxData, "aa")
        row = self.layout.row()
        row.prop(sc.noxData, "engine")
        row = self.layout.row()
        row.operator("wm.buttonrunnoxrenderer")

        if sc.render.layers[0]:
            self.layout.separator()
            row = self.layout.row()
            row.prop(sc.noxData, "override_on")
            row = self.layout.row()
            row.enabled = sc.noxData.override_on
            row.template_ID(sc.render.layers[0], "material_override", new="material.new")

        self.layout.separator()
        
        row = self.layout.row()
        col = row.column()
        col.prop(sc.noxData, "preload_post_on", text='')
        col = row.column()
        col.enabled = sc.noxData.preload_post_on
        col.prop(sc.noxData, "preload_post_file")
        
        row = self.layout.row()
        col = row.column()
        col.prop(sc.noxData, "preload_blend_on", text='')
        col = row.column()
        col.enabled = sc.noxData.preload_blend_on
        col.prop(sc.noxData, "preload_blend_file")

        row = self.layout.row()
        col = row.column()
        col.prop(sc.noxData, "preload_final_on", text='')
        col = row.column()
        col.enabled = sc.noxData.preload_final_on
        col.prop(sc.noxData, "preload_final_file")

        row = self.layout.row()
        row.prop(sc.noxData, "ignore_no_emitter")
        
        self.layout.separator()

        row = self.layout.row()
        row.prop(sc.noxData, "motion_blur_on")
        row = self.layout.row()
        row.enabled = sc.noxData.motion_blur_on
        row.prop(sc.noxData, "motion_blur_duration")
        
        
    @classmethod
    def poll(cls, context):
        return (context.scene.render.engine == 'nox_renderer')

#---------------------------------------------------------------------------------------------------

class NOXBlendLayersPanel(bpy.types.Panel):
    bl_space_type = 'PROPERTIES'
    bl_region_type = 'WINDOW'
    bl_idname = "noxblendlayerspanel"
    bl_label = "NOX Blend Layers"
    bl_context = "render"
    COMPAT_ENGINES = {"nox_renderer"}

    
    def draw(self, context):
        sc = context.scene

        row = self.layout.row()
        row.prop(sc.noxData.blend_layers, "b1")
        row = self.layout.row()
        row.prop(sc.noxData.blend_layers, "b2")
        row = self.layout.row()
        row.prop(sc.noxData.blend_layers, "b3")
        row = self.layout.row()
        row.prop(sc.noxData.blend_layers, "b4")
        row = self.layout.row()
        row.prop(sc.noxData.blend_layers, "b5")
        row = self.layout.row()
        row.prop(sc.noxData.blend_layers, "b6")
        row = self.layout.row()
        row.prop(sc.noxData.blend_layers, "b7")
        row = self.layout.row()
        row.prop(sc.noxData.blend_layers, "b8")
        row = self.layout.row()
        row.prop(sc.noxData.blend_layers, "b9")
        row = self.layout.row()
        row.prop(sc.noxData.blend_layers, "b10")
        row = self.layout.row()
        row.prop(sc.noxData.blend_layers, "b11")
        row = self.layout.row()
        row.prop(sc.noxData.blend_layers, "b12")
        row = self.layout.row()
        row.prop(sc.noxData.blend_layers, "b13")
        row = self.layout.row()
        row.prop(sc.noxData.blend_layers, "b14")
        row = self.layout.row()
        row.prop(sc.noxData.blend_layers, "b15")
        row = self.layout.row()
        row.prop(sc.noxData.blend_layers, "b16")

    @classmethod
    def poll(cls, context):
        return (context.scene.render.engine == 'nox_renderer')

#---------------------------------------------------------------------------------------------------

