import bpy
from mathutils import Vector
#---------------------------------------------------------------------------------------------------

class DrawSelectedBoundsOperator(bpy.types.Operator):
    bl_idname = "wm.buttondrawselectedbounds"
    bl_label = "Draw selected bounds"
    def execute(self, context):
        scene = bpy.context.scene
        if not scene:
            return {'CANCELLED'}
        for ob in scene.objects:
            if ob.select:
                ob.draw_type="BOUNDS"
        return{'FINISHED'}

#---------------------------------------------------------------------------------------------------

class DrawSelectedSolidOperator(bpy.types.Operator):
    bl_idname = "wm.buttondrawselectedsolid"
    bl_label = "Draw selected solid"
    def execute(self, context):
        scene = bpy.context.scene
        if not scene:
            return {'CANCELLED'}
        for ob in scene.objects:
            if ob.select:
                ob.draw_type="SOLID"
        return{'FINISHED'}

#---------------------------------------------------------------------------------------------------

class DrawSelectedTexturedOperator(bpy.types.Operator):
    bl_idname = "wm.buttondrawselectedtextured"
    bl_label = "Draw selected textured"
    def execute(self, context):
        scene = bpy.context.scene
        if not scene:
            return {'CANCELLED'}
        for ob in scene.objects:
            if ob.select:
                ob.draw_type="TEXTURED"
        return{'FINISHED'}

#---------------------------------------------------------------------------------------------------

class PrintBoundBoxOperator(bpy.types.Operator):
    bl_idname = "wm.buttonprintboundbox"
    bl_label = "Print bounding box"
    def execute(self, context):
        scene = bpy.context.scene
        if bpy.context.selected_objects.__len__() < 1:
            return {'CANCELLED'}
        num_ob = bpy.context.selected_objects.__len__()
        tb_minx=100000000000000000000
        tb_miny=100000000000000000000
        tb_minz=100000000000000000000
        tb_maxx=-100000000000000000000
        tb_maxy=-100000000000000000000
        tb_maxz=-100000000000000000000
        for ob in bpy.context.selected_objects:
            print("Checking "+ob.name)
            for v in ob.bound_box:
                vec_in = Vector((v[0], v[1], v[2]))
                vec_out = ob.matrix_world * vec_in
                tb_minx = min(tb_minx, vec_out.x)
                tb_miny = min(tb_miny, vec_out.y)
                tb_minz = min(tb_minz, vec_out.z)
                tb_maxx = max(tb_maxx, vec_out.x)
                tb_maxy = max(tb_maxy, vec_out.y)
                tb_maxz = max(tb_maxz, vec_out.z)
        bbsx = tb_maxx - tb_minx
        bbsy = tb_maxy - tb_miny
        bbsz = tb_maxz - tb_minz
        print("Bound box size: "+str(bbsx)+", "+str(bbsy)+", "+str(bbsz))
        
        return{'FINISHED'}


#---------------------------------------------------------------------------------------------------
